	PROCESSOR 6502
	
; DPCplus ARM code demo
; Last Update:  May 30, 2010
;               Darrell Spice, Jr.
;
; This demo is to show how to use custom ARM code with the DPC+ bankswitching
; format.  The ARM code for this demo is written in C.
;
; This demo assumes you have reviewed the DPCplus demo, so it does not explain
; how to use the DPC+ features.  If you have not reviewed the demo, you can
; obtain it from here:
;       http://www.atariage.com/forums/topic/163495-harmony-dpc-programming/
;
; NOTE: While Stella supports DPC+, it does NOT support custom ARM code.
;       This may change in future builds of Stella (this was written is as of
;       version 3.1.2 of Stella)


TIA_BASE_READ_ADDRESS = $30
	include VCS.H
        include macro.h
        include graphics.h
        include DPCplus.h

; define the bitmap size.  This is based on what the kernel can display.
; The C functions support any size bitmap, these values are passed to it.
BITMAP_WIDTH = 96
BITMAP_HEIGHT = 200

; set OPTIMIZE to 1 to show time remaining below bitmap.  This will cause the
; screen to be taller than normal, so it might cause your TV to roll.
; Time remaining in Vertical Blank is shown on the Left.  Time remaining in
; Overscan is shown on the Right.
OPTIMIZE = 1 	
       
        SEG.U VARS
        ORG $80
        
Frame		ds 1    
TimeLeftOS	ds 1
TimeLeftVB	ds 1
DisplayOS	ds 4
DisplayVB	ds 4


 echo "----",($00FE - *) , "bytes of RAM left"
; $FE - $FF 2 byte stack (used by JSR)

        SEG CODE
        
;----------------------------------------
; Bank 0-4 - ARM user code
;----------------------------------------
; The ARM code must start at the beginning of the ROM space in bank 0.
; Do NOT make space for the 128 DPC+ registers like is done in banks for 6507
; code. 
;----------------------------------------
        ORG $0000
        ; include the custom ARM code.  This must be compiled before compiling
        ; the 6507 code by executing the MAKE command in the custom directory.
        INCBIN custom/bin/custom2.bin
        
        ; Change the $5000s below to control ROM usage:
        ; VALUE		ARM CODE	6507 CODE
        ; $1000		 4K		20K (five  4K banks)
        ; $2000		 8K		16K (four  4K banks)
        ; $3000		12K		12K (three 4K banks)
        ; $4000		16K		 8K (two   4K banks)
        ; $5000		20K		 4K (one   4K bank )
	if (* <= $5000)
	  echo "------",($5000 - *) , "bytes of ARM User Code left"
          ds ($5000 - *)	; pad out remaining space
	else
	  echo "FATAL ERROR - ARM user code exceeds 20K"
	  err
	endif	      
       
;----------------------------------------
; Bank 5 - Custom ARM demo
;----------------------------------------
; This demo draws a 96x200 bitmap using the players.  Both players are set to
; 3 copies w/medium spacing and then shifted back/forth on alternate frames.
; The ARM code calculates a wireframe cube and updates it at 60 fps.
; Across the top are 4 sets of HEX values.  The values, and how to change them
; using the left joystick are:
;	X rotation	= left or right
;	Y rotation	= up or down
;	Z rotation	= Fire + (left or right)
;	Scale		= Fire + (left or right)
;----------------------------------------
        ORG $5000
        RORG $F000
        ds 128 ; makes space for DPC+ registers (arm stub overlaps it)
	
PosObject:   ; A holds X value
        sec  ; X holds object, 0=P0, 1=P1, 2=M0, 3=M1, 4=Ball
        sta WSYNC
DivideLoop
        sbc #15        ; 2
        bcs DivideLoop ; 2    4
        eor #7         ; 2    6
        asl            ; 2    8
        asl            ; 2   10
        asl            ; 2   12
        asl            ; 2   14
        sta.wx HMP0,X  ; 5   19
        sta RESP0,X    ; 4   23 <- set object position
SLEEP12 rts            ; 6    9	; JSR SLEEP12 for a 12 cycle delay
        
InitSystem:
	CLEAN_START
	ldx #$ff
	stx FASTFETCH	; turn off fast fetch mode
	lda #>Digits
	sta DisplayOS+1
	sta DisplayVB+1
	sta DisplayOS+3
	sta DisplayVB+3
		
	; When calling the ARM code, it will always run whatever is in your
	; MAIN() function.  For this demo, I use part of the Display Data bank
	; to pass values.  The first byte tells MAIN() which subroutine you
	; wish to execute. Following bytes are values used by that subroutine.
	; 
	; Prep a DataWriter to save values for use by the ARM code.  In this
	; example the values are stored with a base address of ARMsub:
 	lda #<ARMsub
 	sta DF0LOW
 	lda #>ARMsub
 	sta DF0HI
 	; The first byte denotes the ARM subroutine to run:
	; 	0 = Initialize()
	; 	1 = OverScan()
	; 	2 = VerticalBlank() 	
 	lda #0		; 0 = run ARM subroutine Initialize();
 	sta DF0WRITE
 	; Values after the first depend on the subroutine being called.  For
 	; init we're passing the dimensions of the bitmap and it's starting
 	; address in DisplayData.
 	lda #BITMAP_WIDTH
 	sta DF0WRITE
 	lda #BITMAP_HEIGHT
 	sta DF0WRITE
 	lda #<BitmapData
 	sta DF0WRITE
 	lda #>BitmapData
 	sta DF0WRITE
 	; NOTE: THIS IS MAJOR
 	;       When calling custom ARM code, it's recommend to do so from
 	;	a low address in the 4K bank as the 6507 will be receiving
 	;	NOP instructions while the ARM code is running.  If the
 	;	call is from the end of the 4K bank, there's a good chance
 	;	the 6507 will run beyond the Atari's 4K cartridge space.  If it
 	;	does, your program will crash.
 	lda #$FF
	sta CALLFUNCTION
 		
	jmp KernelDone	; jumping here because OS preps data required for VB
	
	
OverScan:
	; the parameters for the Overscan call are done in KernelDone.  The
	; call is triggered here due to the NOTE: THIS IS MAJOR seen above.
	lda #$FF
	sta CALLFUNCTION
  
	lda INTIM
	sta TimeLeftOS
		
OSwait:
  	sta WSYNC
	lda INTIM
	bpl OSwait

VerticalBlank
	lda #2                            ; VSYNC enable
	sta WSYNC
	sta VSYNC
	sta VBLANK
	lda #$2C
	sta TIM64T
	lda #$07
	sta COLUP0
	sta COLUP1
	lda #%110
	sta NUSIZ0
	sta NUSIZ1
	sta WSYNC
	inc Frame
	sta WSYNC
	lda #0
	sta GRP0
	sta GRP1	
	sta WSYNC
	sta VSYNC 
	
	; prep for call to VerticalBlank()
 	lda #<ARMsub
 	sta DF0LOW
 	lda #>ARMsub
 	sta DF0HI
 	lda #2		; 2 = run ARM subroutine VerticalBlank();
 	sta DF0WRITE
	lda #$FF
	sta CALLFUNCTION
	
	; position sprites, point Data Fetchers to bitmap
	lda Frame
	and #1
	bne Position2
	
Position1:	
	lda #32
	ldx #0
	jsr PosObject
	lda #40
	ldx #1
	jsr PosObject
        sta WSYNC
        sta HMOVE
        lda #<(BitmapData + BITMAP_HEIGHT * 0)	; column 0
        sta DF0LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 0)
        sta DF0HI       
        lda #<(BitmapData + BITMAP_HEIGHT * 1)	; column 1
        sta DF1LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 1)
        sta DF1HI
        lda #<(BitmapData + BITMAP_HEIGHT * 4)	; column 4
        sta DF2LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 4)
        sta DF2HI
        lda #<(BitmapData + BITMAP_HEIGHT * 5)	; column 5
        sta DF3LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 5)
        sta DF3HI
        lda #<(BitmapData + BITMAP_HEIGHT * 8)	; column 8
        sta DF4LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 8)
        sta DF4HI
        lda #<(BitmapData + BITMAP_HEIGHT * 9)	; column 9
        sta DF5LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 9)
        sta DF5HI
        jmp VBend
        
Position2:	
	lda #32+16
	ldx #0
	jsr PosObject
	lda #40+16
	ldx #1
	jsr PosObject
        sta WSYNC
        sta HMOVE      
        lda #<(BitmapData + BITMAP_HEIGHT * 2)	; column 2
        sta DF0LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 2)
        sta DF0HI       
        lda #<(BitmapData + BITMAP_HEIGHT * 3)	; column 3
        sta DF1LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 3)
        sta DF1HI
        lda #<(BitmapData + BITMAP_HEIGHT * 6)	; column 6
        sta DF2LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 6)
        sta DF2HI
        lda #<(BitmapData + BITMAP_HEIGHT * 7)	; column 7
        sta DF3LOW
        lda #>(BitmapData + BITMAP_HEIGHT * 7)
        sta DF3HI
        lda #<(BitmapData + BITMAP_HEIGHT *10)	; column 10
        sta DF4LOW
        lda #>(BitmapData + BITMAP_HEIGHT *10)
        sta DF4HI
        lda #<(BitmapData + BITMAP_HEIGHT *11)	; column 11
        sta DF5LOW
        lda #>(BitmapData + BITMAP_HEIGHT *11)
        sta DF5HI        
        
VBend:        
        
	ldy #200
	lda #0
	sta FASTFETCH	; turn on fast fetch
	
	lda INTIM
	sta TimeLeftVB
	
VblankWait:
	sta WSYNC
	lda INTIM
	bpl VblankWait
	sta WSYNC
	stx VBLANK        ; turn on video output

	; select kernel based on sprite position
	lda Frame
	and #1
	bne Kernel2
	

Kernel1:
	sta WSYNC
	lda #<DF0DATA	; 2
	sta GRP0	; 3  5
	lda #<DF1DATA	; 2  7
	sta GRP1	; 3 10
	SLEEP 21	;21 31
	lda #<DF2DATA	; 2 33
	sta GRP0	; 3 36
	lda #<DF3DATA	; 2 38
	sta GRP1	; 3 41
	SLEEP 2		; 2 43
	lda #<DF4DATA	; 2 45
	sta GRP0	; 3 48
	lda #<DF5DATA	; 2 50
	sta GRP1	; 3 53
	dey		; 2 55
	bne Kernel1	; 2 57
	beq KernelDone	; 3 60
	
Kernel2:
	sta WSYNC
	lda #<DF0DATA
	sta GRP0
	lda #<DF1DATA
	sta GRP1
	SLEEP 27
	lda #<DF2DATA
	sta GRP0
	lda #<DF3DATA
	sta GRP1
	SLEEP 2
	lda #<DF4DATA
	sta GRP0
	lda #<DF5DATA
	sta GRP1
	dey
	bne Kernel2	
		
KernelDone:
 IF OPTIMIZE = 1
 	; display time remaining
 	ldx #0
 	stx GRP0
 	stx GRP1
 	sta WSYNC
 	ldx #%100 
 	stx NUSIZ0
 	stx NUSIZ1
 	lda TimeLeftVB
 	and #$0f
 	tax
 	lda Times7,x
 	sta DisplayVB+2
 	lda TimeLeftVB
 	lsr
 	lsr
 	lsr
 	sta RESP0
 	sta RESP1
 	lsr
 	tax
 	lda Times7,x
 	sta DisplayVB
 	
 	lda TimeLeftOS	;  3 11
 	and #$0f	;  2 13
 	tax
 	lda Times7,x
 	sta DisplayOS+2
 	lda TimeLeftOS
 	lsr
 	lsr
 	lsr
 	lsr
 	tax
 	lda Times7,x
 	sta DisplayOS
 	
 	ldy #6
TimeLoop:
	sta WSYNC
	lda #$84		; 2
	sta COLUP0		; 3  5 
	sta COLUP1		; 3  8
	lda (DisplayVB),y	; 5 13
	sta GRP0		; 3 16
	lda (DisplayVB+2),y	; 5 21
	sta GRP1		; 3 24
	SLEEP 10
	lda #$44		; 2 26
	sta COLUP0		; 3 29
	sta COLUP1		; 3 31
	lda (DisplayOS),y	; 5 36
	sta GRP0		; 3 39
	lda (DisplayOS+2),y	; 5 44
	sta GRP1		; 3 48
	dey
	bpl TimeLoop
	 
 ENDIF
	ldy #2
 	ldx #$18	; prep overscan delay
 	sty WSYNC
 	sty VBLANK
 	stx TIM64T	; set overscan delay
	lda #$FF
	sta FASTFETCH	; turn off fast fetch
 	
 	
 	
 	; prep for call to VerticalBlank()
 	lda #<ARMsub
 	sta DF0LOW
 	lda #>ARMsub
 	sta DF0HI
 	lda #1		; 1 = run ARM subroutine OverScan();
 	sta DF0WRITE
 	; pass the PIA and TIA registers to the ARM code
 	lda SWCHA
 	sta DF0WRITE
 	lda SWCHB
 	sta DF0WRITE
 	lda CXM0P
 	sta DF0WRITE
 	lda CXM1P
 	sta DF0WRITE
 	lda CXP0FB
 	sta DF0WRITE
 	lda CXP1FB
 	sta DF0WRITE
 	lda CXM0FB
 	sta DF0WRITE
 	lda CXM1FB
 	sta DF0WRITE
 	lda CXBLPF
 	sta DF0WRITE
 	lda CXPPMM
 	sta DF0WRITE
 	lda INPT4
 	sta DF0WRITE
 	lda INPT5
 	sta DF0WRITE
 	jmp OverScan

 	align 256
 	; upside down digits used for traditional Atari Player code
Digits:
       .byte zz____X___ ; ZERO
       .byte zz__XX_X__
       .byte zz_XX___X_
       .byte zz_X____X_
       .byte zz_X___XX_
       .byte zz__X_XX__
       .byte zz___X____

       .byte zz____X___ ; ONE
       .byte zz____XX__
       .byte zz____XX__
       .byte zz__X_XX__
       .byte zz___XXX__
       .byte zz____XX__
       .byte zz_____X__

       .byte zz____XXX_ ; TWO
       .byte zz_XXXX___
       .byte zz__X_____
       .byte zz___XX___
       .byte zz_____X__
       .byte zz__X___X_
       .byte zz___XXX__

       .byte zz___XX___ ; THREE
       .byte zz__XX_X__
       .byte zz_XX___X_
       .byte zz______X_
       .byte zz____XX__
       .byte zz__X___X_
       .byte zz___XXX__

       .byte zz____X___ ; FOUR
       .byte zz____XX__
       .byte zz_XXXXX__
       .byte zz__X_XX__
       .byte zz___XXX__
       .byte zz____XX__
       .byte zz_____X__


       .byte zz__XXX___ ; FIVE
       .byte zz_X___X__
       .byte zz______X_
       .byte zz_X___X__
       .byte zz_XXXX___
       .byte zz_X______
       .byte zz_XXXXXX_

       .byte zz___XX___ ; SIX
       .byte zz__XX_X__
       .byte zz_XX___X_
       .byte zz_X_XXX__
       .byte zz_X______
       .byte zz__X__X__
       .byte zz___XX___

       .byte zz___X____ ; SEVEN
       .byte zz___X____
       .byte zz___X____
       .byte zz____X___
       .byte zz_____X__
       .byte zz______X_
       .byte zz_XXXXXX_

       .byte zz___XX___ ; EIGHT
       .byte zz__XX_X__
       .byte zz_XX__XX_
       .byte zz_XX_XX__
       .byte zz___XXX__
       .byte zz__XX__X_
       .byte zz___XXX__

       .byte zz___XX___ ; NINE
       .byte zz__X__X__
       .byte zz______X_
       .byte zz__XXX_X_
       .byte zz_X___XX_
       .byte zz__X_XX__
       .byte zz___XX___

       .byte zz_X_____X ; A (TEN)
       .byte zz_X_____X
       .byte zz_X_____X
       .byte zz_XXXXXXX
       .byte zz__X___X_
       .byte zz___X_X__
       .byte zz____X___

       .byte zz_XXXX___ ; B (ELEVEN)
       .byte zz_X___X__
       .byte zz_X___X__
       .byte zz_XXXX___
       .byte zz_X___X__
       .byte zz_X___X__
       .byte zz_XXXX___

       .byte zz__XXX___ ; C (TWELVE)
       .byte zz_X___X__
       .byte zz_X______
       .byte zz_X______
       .byte zz_X______
       .byte zz_X___X__
       .byte zz__XXX___

       .byte zz_XXXX___ ; D (THIRTEEN)
       .byte zz_X___X__
       .byte zz_X____X_
       .byte zz_X____X_
       .byte zz_X____X_
       .byte zz_X___X__
       .byte zz_XXXX___

       .byte zz_XXXXXX_ ; E (FORTEEN)
       .byte zz_X______
       .byte zz_X______
       .byte zz_XXXX___
       .byte zz_X______
       .byte zz_X______
       .byte zz_XXXXXX_

       .byte zz_X______ ; F (FIFTEEN)
       .byte zz_X______
       .byte zz_X______
       .byte zz_XXXX___
       .byte zz_X______
       .byte zz_X______
       .byte zz_XXXXXX_

       .byte 0,0,0,0,0,0,0 ; blank

Times7  .byte $0,7,14,21,28,35,42,49,56,63,70,77,84,91,98,105 

	echo "------",($FF00 - *) , "bytes of BANK 5b ROM left"

	ORG $5F00
	RORG $FF00
	; ARMcodeDigits are used in the ARM subroutine OverScan() to display the
	; current X, Y, Z and Scale values.
ARMcodeDigit0:
        .byte zz__XXXX__;--
        .byte zz_XX_XXX_;--
        .byte zz_X___XX_;--
        .byte zzXX___XX_;--
        .byte zzXX___XX_;--
        .byte zzXX__XX__;--
        .byte zzXXX_XX__;--
        .byte zz_XXXX___;--
        
ARMcodeDigit1:
        .byte zz____X___;--
        .byte zz___XX___;--
        .byte zz__XXX___;--
        .byte zz_X_XX___;--
        .byte zz___XX___;--
        .byte zz___XX___;--
        .byte zz___XX___;--
        .byte zz_XXXXXX_;--
        
ARMcodeDigit2:
        .byte zz__XXXX__;--
        .byte zz_XX_XXX_;--
        .byte zz_____XX_;--
        .byte zz_____XX_;--
        .byte zz____XX__;--
        .byte zz___XX___;--
        .byte zz_XXX____;--
        .byte zzXXXXXXX_;--
        
ARMcodeDigit3:
        .byte zz__XXXX__;--
        .byte zz_XX__XX_;--
        .byte zz____XX__;--
        .byte zz__XXX___;--
        .byte zz___XXX__;--
        .byte zz____XX__;--
        .byte zz___XXX__;--
        .byte zzXXXXX___;--
        
ARMcodeDigit4:
        .byte zz_____X__;--
        .byte zz____XX__;--
        .byte zz___XXX__;--
        .byte zz__X_XX__;--
        .byte zz_X__XX__;--
        .byte zzXXXXXXX_;--
        .byte zz____XX__;--
        .byte zz____XX__;--
        
ARMcodeDigit5:
        .byte zz__XXXXX_;--
        .byte zz_XX_____;--
        .byte zz_XXX____;--
        .byte zz_XXXXX__;--
        .byte zz____XX__;--       
        .byte zz_____X__;--
        .byte zz____XX__;--
        .byte zzXXXXX___;--
        
ARMcodeDigit6:
        .byte zz__XXXXX_;--
        .byte zz_XX_____;--
        .byte zz_X______;--
        .byte zzXXXXX___;--
        .byte zzXXXXXX__;--
        .byte zzXX__XX__;--
        .byte zzXX__XX__;--
        .byte zzXXXXX___;--
        
ARMcodeDigit7:
        .byte zz_XXXXXX_;--
        .byte zz____XX__;--
        .byte zz___XX___;--
        .byte zz___XX___;--
        .byte zz__XX____;--
        .byte zz__XX____;--
        .byte zz_XX_____;--
        .byte zzXX______;--
        
ARMcodeDigit8:
        .byte zz__XXXX__;--
        .byte zz_XX__XX_;--
        .byte zz_XX_XX__;--
        .byte zz__XXX___;--
        .byte zz_XXXXX__;--       
        .byte zzXX___X__;--
        .byte zzXX__XX__;--
        .byte zzXXXXX___;--

ARMcodeDigit9:
        .byte zz__XXXX__;--
        .byte zz_XX__XX_;--
        .byte zz_XX__XX_;--
        .byte zz_XX__XX_;--
        .byte zz_XXXXXX_;--
        .byte zz_____X__;--
        .byte zz____XX__;--
        .byte zzXXXXX___;--
        
ARMcodeDigitA:
        .byte zz___XX___ ; 7
        .byte zz__X__X__ ; 6
        .byte zz_X____X_ ; 5
        .byte zz_XXXXXX_ ; 4
        .byte zz_X____X_ ; 0
        .byte zz_X____X_ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_X____X_ ; 3

ARMcodeDigitB:
        .byte zz_XXXXX__ ; 0
        .byte zz_X____X_ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_XXXXX__ ; 4
        .byte zz_X____X_ ; 3
        .byte zz_X____X_ ; 5
        .byte zz_X____X_ ; 6
        .byte zz_XXXXX__ ; 7
ARMcodeDigitC:
        .byte zz__XXXX__ ; 0
        .byte zz_X____X_ ; 1
        .byte zz_X______ ; 2
        .byte zz_X______ ; 3
        .byte zz_X______ ; 4
        .byte zz_X______ ; 5
        .byte zz_X____X_ ; 6
        .byte zz__XXXX__ ; 7
ARMcodeDigitD:
        .byte zz_XXXXX__ ; 0
        .byte zz_X____X_ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_X____X_ ; 3
        .byte zz_X____X_ ; 4
        .byte zz_X____X_ ; 5
        .byte zz_X____X_ ; 6
        .byte zz_XXXXX__ ; 7
        
ARMcodeDigitE:
        .byte zz_XXXXXX_ ; 7
        .byte zz_X______ ; 6
        .byte zz_X______ ; 5
        .byte zz_XXXXXX_ ; 4
        .byte zz_X______ ; 3
        .byte zz_X______ ; 2
        .byte zz_X______ ; 1
        .byte zz_XXXXXX_ ; 0

ARMcodeDigitF:
        .byte zz_XXXXXX_ ; 7        
        .byte zz_X______ ; 6
        .byte zz_X______ ; 5
        .byte zz_XXXXXX_ ; 4
        .byte zz_X______ ; 3
        .byte zz_X______ ; 2
        .byte zz_X______ ; 1
        .byte zz_X______ ; 0

        ; we're not bankswitching in this demo, but we need to "block off" the
        ; hot spots so we don't accidently trigger a bank switch.
	echo "------",($FFF6 - *) , "bytes of BANK 5c ROM left"
        ORG $5FF6
        RORG $FFF6
SelectBank0 .byte $00
SelectBank1 .byte $00
SelectBank2 .byte $00
SelectBank3 .byte $00
SelectBank4 .byte $00
SelectBank5 .byte $00
;        .word InitSystem ; NMI - overlaps with SelectBank4 and 5
        .word InitSystem ; RESET
        .word InitSystem ; IRQ
        
        
;----------------------------------------
; Display Data
;----------------------------------------
; The Display Data bank is copied into RAM when DPC+ initializes the cartridge.
; This allows us to manipulate the data during run-time, but have a known
; starting state when the Atari is first turned on.
;
; Unlike normal Atari VCS/2600 sprite definitions, the sprite data in the
; Display Data bank is stored right-side-up.
;
;----------------------------------------
        ORG $6000
        RORG $0000       
; NOTE the use of RORG $0000 instead of $F000
 
; In this demo, the first set of bytes in Display Data are used to pass values
; to the ARM code
ARMsub		ds 1	; 0 for init, 1 for Over Scan, 2 for Vertical Blank
ARMswcha	ds 1	; controller input to ARM
ARMswchb	ds 1	; console switches to ARM
ARMcxm0p	ds 1	; pass TIA Read registers to ARM
ARMcxm1p	ds 1
ARMcxp0fb	ds 1
ARMcxp1fb	ds 1
ARMcxm0fb	ds 1
ARMcxm1fb	ds 1
ARMcxblpf	ds 1
ARMcxppmm	ds 1
ARMinpt4	ds 1
ARMinpt5	ds 1

BitmapData	ds BITMAP_WIDTH * BITMAP_HEIGHT / 8
       
	if (* <= $1000)
	  echo "------",($1000 - *) , "bytes of Display Data ROM left"
          ds ($1000 - *)	; pad out remaining space in Display Data ROM
	else
	  echo "FATAL ERROR - Display Data exceeds 4K"
	  err
	endif	

        include DPC_frequencies.h       
